<?php

namespace Drupal\server_sent_events\Entity;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityPublishedTrait;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\user\UserInterface;

/**
 * Defines the Server Sent Event Entity entity.
 *
 * @ingroup server_sent_events
 *
 * @ContentEntityType(
 *   id = "server_event_entity",
 *   label = @Translation("Server Sent Event Entity"),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\server_sent_events\SSEventEntityListBuilder",
 *     "views_data" = "Drupal\server_sent_events\Entity\SSEventEntityViewsData",
 *
 *     "access" = "Drupal\server_sent_events\SSEventEntityAccessControlHandler",
 *   },
 *   base_table = "server_event_entity",
 *   translatable = FALSE,
 *   admin_permission = "administer server sent event entity entities",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "type",
 *     "uuid" = "uuid",
 *     "uid" = "user_id",
 *     "langcode" = "langcode",
 *     "published" = "status",
 *   },
 * )
 */
class SSEventEntity extends ContentEntityBase implements SSEventEntityInterface {

  use EntityPublishedTrait;

  /**
   * {@inheritdoc}
   */
  public static function preCreate(EntityStorageInterface $storage_controller, array &$values) {
    parent::preCreate($storage_controller, $values);
    $values += [
      'user_id' => \Drupal::currentUser()->id(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(EntityStorageInterface $storage_controller, $update = TRUE) {
    //todo test this logic
    parent::postSave($storage_controller);
    $this->setOwnerId(\Drupal::currentUser()->id());
    $this->dispatch();
    if($this->get('recipients')->isEmpty() && $update){
      //todo delete logic
      $this->logger()->notice('Updated entity without recipients , schedule for delete');
      $this->delete();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return $this->get('type')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setName($name) {
    $this->set('type', $name);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwner() {
    return $this->get('user_id')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwnerId() {
    return $this->get('user_id')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwnerId($uid) {
    $this->set('user_id', $uid);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwner(UserInterface $account) {
    $this->set('user_id', $account->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function logger() {
    if(!isset($this->logger)){
      $this->logger = \Drupal::logger('server_sent_events');
    }
    return $this->logger;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    // Add the published field.
    $fields += static::publishedBaseFieldDefinitions($entity_type);

    $fields['user_id'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Authored by'))
      ->setDescription(t('The user ID of author of the Server Sent Event Entity entity.'))
      ->setRevisionable(TRUE)
      ->setSetting('target_type', 'user')
      ->setSetting('handler', 'default')
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'author',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['channel'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Channel'))
      ->setDescription(t('The server sent event channel'))
      ->setRevisionable(FALSE)
      ->setSetting('target_type', 'events_channel')
      ->setSetting('handler', 'default')
      ->setRequired(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE);


    $fields['recipients'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Recipients'))
      ->setDescription(t('The target subscriptions to receive this event.'))
      ->setRevisionable(TRUE)
      ->setSetting('target_type', 'channel_subscription')
      ->setSetting('handler', 'default')
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'author',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setCardinality(FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED);

    $fields['type'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Type'))
      ->setDescription(t('The name of the Server Sent Event Entity entity.'))
      ->setSettings([
        'max_length' => 50,
        'text_processing' => 0,
      ])
      ->setDefaultValue('message')
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => -4,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -4,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setRequired(TRUE);

    $fields['body'] =BaseFieldDefinition::create('string_long')
      ->setLabel(t('Text (formatted, long)'))
      ->setDescription(t('Test formatted text.'))
      ->setDisplayOptions('form', [
        'type' => 'string_textarea',
        'weight' => 6,
        'rows' => 6,
      ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['status']->setDescription(t('A boolean indicating whether the Server Sent Event Entity is published.'))
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
        'weight' => -3,
      ]);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created'))
      ->setDescription(t('The time that the entity was created.'));

    return $fields;
  }

  public function deliverEvent(ChannelSubscription $subscription){
    /*$subscriptions = $this->get('recipients')->getValue();
    $sids = array_column($subscriptions,'target_id');
    $delta = array_search($subscription->id(),$sids);
    if($delta){
      $this->get('recipients')->removeItem($delta);
    }*/
    $recipients = [];
    foreach($this->get('recipients') as $delta => $recipientItem){
      if($recipientItem->target_id != $subscription->id()){
        $recipients[] = ['target_id' => $recipientItem->target_id];
      }
    }
    $this->set('recipients' , $recipients);
  }

  public function dispatch(){
    //TODO dispatch event
    $this->logger()->notice('Event dispatched');
  }

  public function send(){
    $data = $this->body[0]->value;
    $event = "\nevent: {$this->type[0]->value}\nid: {$this->id()}\ndata: $data\n\n";
    echo $event;
    ob_flush();
    flush();
  }


}
